//
//  ContentView.swift
//  BTSP02a
//
//  Created by 富樫 豊彦 on 2024/03/05.
//

import SwiftUI
import Foundation
import CoreBluetooth

struct ContentView: View {
    @StateObject var btm = BTmanage()

    var body: some View {
        Text("BTSP02 Remote Controller")
            .font(.title)
        Text(btm.msgs)
        VStack {
            Text("Main Volume")
            HStack {
                Slider(value: $btm.mv, in: 0 ... 100, step: 1,
                        onEditingChanged: { editing in
                            btm.sendMainVolume()
                        }
                )
                Text("\(Int(btm.mv))%")
            }
            Text("Input Gain")
            HStack {
                Text("A")
                Slider(value: $btm.g1, in: 0 ... 200, step: 1,
                       onEditingChanged: { editing in
                           btm.sendGain1()
                       }
                )
                Text("\(Int(btm.g1))%")
            }
            HStack {
                Text("B")
                Slider(value: $btm.g2, in: 0 ... 200, step: 1,
                       onEditingChanged: { editing in
                           btm.sendGain2()
                       }
                )
                Text("\(Int(btm.g2))%")
            }
            HStack {
                Text("BT")
                Slider(value: $btm.g3, in: 0 ... 200, step: 1,
                       onEditingChanged: { editing in
                           btm.sendGain3()
                       }
                )
                Text("\(Int(btm.g3))%")
            }
            Text("Input Balance")
            HStack {
                Text("A")
                Slider(value: $btm.b1, in: -100 ... 100, step: 1,
                       onEditingChanged: { editing in
                           btm.sendBalance1()
                       }
                )
                Text("\(Int(btm.b1))%")
                   .onLongPressGesture {
                       btm.b1 = 0
                       btm.sendBalance1()
                   }
            }
            HStack {
                Text("B")
                Slider(value: $btm.b2, in: -100 ... 100, step: 1,
                       onEditingChanged: { editing in
                           btm.sendBalance2()
                       }
                )
                Text("\(Int(btm.b2))%")
                    .onLongPressGesture {
                        btm.b2 = 0
                        btm.sendBalance2()
                    }
            }
            HStack {
                Text("BT")
                Slider(value: $btm.b3, in: -100 ... 100, step: 1,
                       onEditingChanged: { editing in
                           btm.sendBalance3()
                       }
                )
                Text("\(Int(btm.b3))%")
                    .onLongPressGesture {
                        btm.b3 = 0
                        btm.sendBalance3()
                    }
            }
            
        }
        .padding()
        Text("(c)2024 Toyohiko TOGASHI")
            .font(.caption2)
    }
}

// --- Bluetooth task ---

class BTmanage: NSObject, ObservableObject, CBCentralManagerDelegate, CBPeripheralDelegate {
    let sv = CBUUID(string: "1815") // GATT Automation IO service
    let cr = CBUUID(string: "6FE48AFD-0000-4A6E-A5A3-10160A6E5934")
    let c2 = CBUUID(string: "6FE48AFD-0002-4A6E-A5A3-10160A6E5934")
    let c3 = CBUUID(string: "6FE48AFD-0003-4A6E-A5A3-10160A6E5934")
    let c4 = CBUUID(string: "6FE48AFD-0004-4A6E-A5A3-10160A6E5934")
    let c5 = CBUUID(string: "6FE48AFD-0005-4A6E-A5A3-10160A6E5934")
    let c6 = CBUUID(string: "6FE48AFD-0006-4A6E-A5A3-10160A6E5934")
    let c7 = CBUUID(string: "6FE48AFD-0007-4A6E-A5A3-10160A6E5934")
    var cm: CBCentralManager!
    var dp: CBPeripheral!           // discoverd peripheral
    var dc: CBCharacteristic!       // discoverd characteristic
    var d2: CBCharacteristic!
    var d3: CBCharacteristic!
    var d4: CBCharacteristic!
    var d5: CBCharacteristic!
    var d6: CBCharacteristic!
    var d7: CBCharacteristic!
    @Published var msgs = ""
    @Published var mv = 10.0    // Main
    @Published var g1 = 100.0   // Gain #1
    @Published var b1 = 0.0     // Balance #1
    @Published var g2 = 100.0
    @Published var b2 = 0.0
    @Published var g3 = 100.0
    @Published var b3 = 0.0
    
    override init(){
        super.init()
        cm = CBCentralManager(delegate: self, queue: nil, options: [CBCentralManagerOptionShowPowerAlertKey: true])
        print("BT init")
    }
    
    // --- CBCentralManagerDelegate ---
    
    func centralManagerDidUpdateState(_ m: CBCentralManager) {
        print("BT changed state \(m.state)")
        if m.state == .poweredOn {
            cm.scanForPeripherals(withServices: [sv], options: nil)
            msgs = "Scan start..."
        } else {
            msgs = "Environment error \(m.state.rawValue) ! "
        }
    }
    
    func centralManager(_ central: CBCentralManager, didDiscover p: CBPeripheral, advertisementData: [String : Any], rssi r: NSNumber) {
        print("BT didDiscover \(p) RSSI=\(r)")
        let m = p.name!
        if ((m.prefix(4) == "BTSP") ||
            (m.prefix(5) == "Audio")) {
            if dp != p {
                dp = p
                cm.connect(p, options: nil)
                msgs = "Found " + m + " at " + r.stringValue + "[db]"
            }
        }
    }
    
    func centralManager(_ central: CBCentralManager, didConnect p: CBPeripheral) {
        print("BT didConnect \(p)")
        msgs = "Connected " + p.name!
        cm.stopScan()
        p.delegate = self
        p.discoverServices([sv])    // Control to CBPeripheralDelegate
    }
    
    func centralManager(_ central: CBCentralManager, didFailToConnect p: CBPeripheral, error e: Error?) {
        print("BT didFailToConnect \(p) error=\(String(describing: e?.localizedDescription))")
        msgs = String(describing: e?.localizedDescription)
    }
    
    func centralManager(_ central: CBCentralManager, didDisconnectPeripheral p: CBPeripheral, error e: Error?) {
        print("BT didDisconnectPeripheral \(p) error=\(String(describing: e?.localizedDescription))")
        if e == nil {
            dp = nil
            dc = nil
            d2 = nil
            d3 = nil
            d4 = nil
            d5 = nil
            d6 = nil
            d7 = nil
            cm.scanForPeripherals(withServices: [sv], options: nil)
            msgs = "Searching..."
        } else {
            msgs = String(describing: e?.localizedDescription)
        }
    }
    
    // --- CBPeripheralDelegate ---
    
    func peripheral(_ peripheral: CBPeripheral, didDiscoverServices e: Error?) {
        print("BT didDiscoverServices")
        if e == nil {
            for s in peripheral.services! {
                //              peripheral.discoverCharacteristics([cr, c2, c3, c4, c5, c6, c7], for: s)
                peripheral.discoverCharacteristics(nil, for: s)
                print("sv=\(s)")
            }
        } else {
            msgs = String(describing: e?.localizedDescription)
        }
    }
    
    func peripheral(_ peripheral: CBPeripheral, didDiscoverCharacteristicsFor s: CBService, error e: Error?) {
        print("BT didDiscoverCharacteristicsFor \(s)")
        if e == nil {
            for c in s.characteristics! {
                //              print("char=\(c)")
                peripheral.setNotifyValue(true, for: c)
                peripheral.readValue(for: c)
                switch c.uuid {
                case cr:
                    dc = c
                    break
                case c2:
                    d2 = c
                    break
                case c3:
                    d3 = c
                    break
                case c4:
                    d4 = c
                    break
                case c5:
                    d5 = c
                    break
                case c6:
                    d6 = c
                    break
                case c7:
                    d7 = c
                    break
                default:
                    print("Unmatch \(c)")
                    break
                }
            }
        } else {
            let m = String(describing: e?.localizedDescription)
            print("Error=\(m)")
        }
    }
    
    func peripheral(_ peripheral: CBPeripheral, didUpdateValueFor c: CBCharacteristic, error e: Error?) {
        print("BT didUpdateValueFor \(c)")
        if e == nil {
            let v = c.value?.withUnsafeBytes { $0.load( as: Int16.self ) }
            switch c.uuid {
            case cr:
                mv = Double(v ?? 0)
                break
            case c2:
                g1 = Double(v ?? 0)
                break
            case c3:
                b1 = Double(v ?? 0)
                break
            case c4:
                g2 = Double(v ?? 0)
                break
            case c5:
                b2 = Double(v ?? 0)
                break
            case c6:
                g3 = Double(v ?? 0)
                break
            case c7:
                b3 = Double(v ?? 0)
                break
            default:
                print("Unmatch")
                break
            }
        } else {
            let m = String(describing: e?.localizedDescription)
            print("Error=\(m)")
        }
    }
    
    func peripheral(_ peripheral: CBPeripheral, didUpdateNotificationStateFor c: CBCharacteristic, error e: Error?) {
        print("BT didUpdateNotificationStateFor")
        if e != nil {
            let m = String(describing: e?.localizedDescription)
            print("Error=\(m) \(c)")
        }
    }
    
    func peripheralIsReady(toSendWriteWithoutResponse p: CBPeripheral) {
        print("BT toSendWriteWithoutResponse")
        print("char=\(p)")
    }
    
    
    // --- my procedure ---
    
    func sendMainVolume(){
        if (dc != nil) {
            var v = Int16(mv)
            let d = Data(bytes: &v, count: MemoryLayout.size(ofValue: v))
            dp.writeValue(d, for: dc, type: CBCharacteristicWriteType.withResponse)
        }
    }
    func sendGain1(){
        if (d2 != nil) {
            var v = Int16(g1)
            let d = Data(bytes: &v, count: MemoryLayout.size(ofValue: v))
            dp.writeValue(d, for: d2, type: CBCharacteristicWriteType.withResponse)
        }
    }
    func sendBalance1(){
        if (d3 != nil) {
            var v = Int16(b1)
            let d = Data(bytes: &v, count: MemoryLayout.size(ofValue: v))
            dp.writeValue(d, for: d3, type: CBCharacteristicWriteType.withResponse)
        }
    }
    func sendGain2(){
        if (d4 != nil) {
            var v = Int16(g2)
            let d = Data(bytes: &v, count: MemoryLayout.size(ofValue: v))
            dp.writeValue(d, for: d4, type: CBCharacteristicWriteType.withResponse)
        }
    }
    func sendBalance2(){
        if (d5 != nil) {
            var v = Int16(b2)
            let d = Data(bytes: &v, count: MemoryLayout.size(ofValue: v))
            dp.writeValue(d, for: d5, type: CBCharacteristicWriteType.withResponse)
        }
    }
    func sendGain3(){
        if (d6 != nil) {
            var v = Int16(g3)
            let d = Data(bytes: &v, count: MemoryLayout.size(ofValue: v))
            dp.writeValue(d, for: d6, type: CBCharacteristicWriteType.withResponse)
        }
    }
    func sendBalance3(){
        if (d7 != nil) {
            var v = Int16(b3)
            let d = Data(bytes: &v, count: MemoryLayout.size(ofValue: v))
            dp.writeValue(d, for: d7, type: CBCharacteristicWriteType.withResponse)
        }
    }
}
#Preview {
    ContentView()
}
